#!/usr/bin/env python

__author__ = 'Jeffrey Knockel'
__copyright__ = 'Copyright 2013 Jeffrey Knockel'
__license__ = 'GPL v2'

import socket
import sys

from dpkt import Packet
from dpkt.icmp6 import ICMP6

from nslookup6 import lookup_or_die
from sequence import ICMP6Sequence
from sniffing import fragmentbuf6, start_sniffing6 as start_sniffing

def worker(sniffer, address, mtu):
    seq = ICMP6Sequence(address, iface=sniffer.iface)
    timeout = sniffer.timeout()
    while True:
        timeout.add_time(1.0)
        request = seq.next_packet_testing_mtu(mtu)
        print 'Sending %d...' % request.extension_hdrs[44].id
        send_icmp(sniffer, request)
        received = False
        while True:
            p = receive_icmp(sniffer, request, timeout, True)
            if p is None:
                if not received:
                    print '*'
                break
            else:
                received = True
                frag = p.extension_hdrs[44]
                if frag is not None:
                    print '%d+%d' % (frag.id,
                                     frag.frag_off_resv_m & 0xfff8)
                else:
                    print '*+0'
        sys.stdout.flush()

def send_icmp(sniffer, ps):
    if isinstance(ps, Packet):
        ps = [ps]
    for p in ps:
        fragmented = fragmentbuf6(p, 1280)
        sniffer.sendbuf(fragmented, p.dst)

def receive_icmp(sniffer, sents, timeout, fragments=False):
    if isinstance(sents, Packet):
        sents = [sents]
    while True:
        p = sniffer.sniff(timeout)
        if p is None:
            return None
        p_load = None
        for sent in sents:
            sent_icmp = sent.data
            sent_echo = sent_icmp.data
            if p.src == sent.dst:
                p_icmp = p.data
                p_frag = p.extension_hdrs[44]
                if p_frag is None or p_frag.frag_off_resv_m & 0xfff8 == 0:
                    if type(p_icmp) is ICMP6 and type(p_icmp.data) is ICMP6.Echo:
                        p_echo = p_icmp.data
                        if p_echo.id == sent_echo.id and \
                                p_echo.seq == sent_echo.seq:
                            # ignore data for first fragment
                            return p
                elif fragments and p.p == socket.IPPROTO_ICMPV6:
                    # only check ICMP data
                    if p_load is None:
                        p_load = str(p_icmp)[:8]
                    sent_load = str(sent_echo.data)
                    sent_load = sent_load[-(((len(sent_load) - 1) % 8) + 1):]
                    # assume that data is 8-byte periodic
                    if p_load == sent_load:
                        return p

def usage(code=1):
    sys.stderr.write('Usage: %s HOST MTU\n' % sys.argv[0])
    exit(code)

def main():
    try:
        if len(sys.argv) != 3:
            usage()
        host = sys.argv[1]
        try:
            mtu = int(sys.argv[2])
        except ValueError:
            usage()
        addr = lookup_or_die(host)[0]
        start_sniffing(worker, addr, mtu)
    except KeyboardInterrupt:
        sys.stderr.write('Received SIGINT\n')

if __name__ == '__main__':
    main()
